<?php
namespace Ep\Gath\WordPress;

use Ep\Core\UserProviderInterface;
use WP_Post;
use WP_Query;

/**
 * Class ProductProvider
 *
 * @package Ep\Gath\WordPress
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-webshop
 */
class ProductProvider
{
	/**
	 * ProductProvider constructor.
	 *
	 * @param GathService $gath
	 * @param UserProviderInterface $users
	 */
	public function __construct(GathService $gath, UserProviderInterface $users)
	{
		parent::__construct($gath, $users);

		add_filter('pre_get_posts', function(WP_Query $query) {
			$query->set('no_found_rows', false);
		});

		add_filter('posts_pre_query', [$this, 'getPosts'], 10, 2);
		add_filter('woocommerce_data_stores', [$this, 'dataStores'], 10, 1);
/*
		add_filter('woocommerce_is_purchasable', function() {
			return true;
		}, 99);
*/

		add_filter('woocommerce_product_subcategories_hide_empty', function() {
			return false;
		});

		add_filter('the_title', [$this, 'filterProductTitle'], 10, 2);

		add_filter('woocommerce_add_to_cart_validation', function($valid, $productId) {
			$this->getProduct($productId);
			return $valid;
		}, 10, 2);

		add_filter('wp_get_object_terms', [$this, 'getObjectTerms'], 10, 4);
		add_filter('get_the_terms', [$this, 'getTerms'], 10, 3);

		add_action('wp_ajax_flatsome_quickview', [$this, 'preloadPost'], 1);
		add_action('wp_ajax_nopriv_flatsome_quickview', [$this, 'preloadPost'], 1);
	}

    /**
     * @param $null
     * @param WP_Query $query
     *
     * @return WP_Post[]
     * @throws \Exception
     */
	public function getPosts($null, WP_Query $query)
	{
        switch ($query->query_vars['orderby'] ?? '') {
            case 'price':
                $order = $query->query_vars['order'] === 'DESC'
                    ? GathService::SORT_PRICE_DESC
                    : GathService::SORT_PRICE_ASC;
                break;
            case 'date ID':
            default:
                $order = GathService::SORT_DATE_DESC;
                break;
        }

        if ($genericArticleIds = $query->get('moto_genids')) {
            $ids = array_map('absint', explode(';', $genericArticleIds));
            $motoId = intval($query->get('moto_id'));

            if (0 === $motoId) {
                return [];
            }

            $query->set('no_found_rows', false);
            $limit = $query->query_vars['posts_per_page'];
            $page = max(1, intval($query->query_vars['paged']));
            $offset = ($page - 1) * $limit;

            $products = $this->gath->getMotoProducts($motoId, $ids, $order, $limit, $offset);

            $query->found_posts = $this->gath->getFoundRows();
            $query->max_num_pages = ceil($query->found_posts / $limit);

            return $this->loadPosts($products);
        }

		foreach ($query->meta_query->queries as $metaQuery) {
			if (isset($metaQuery['key']) && in_array($metaQuery['key'], ['_tecdoc_key', '_tecDoc_key'])) {
				$tecDocPairs = [];

				foreach ($metaQuery['value'] as $key) {
					[$brandNo, $articleNo] = explode('|', $key);
					$tecDocPairs[] = [
						'BrandNo' => intval($brandNo),
						'ArticleNo' => $articleNo,
					];
				}

				$query->set('no_found_rows', false);
				$limit = $query->query_vars['posts_per_page'];
				$page = max(1, intval($query->query_vars['paged']));
				$offset = ($page - 1) * $limit;

				$products = $this->gath->getTecDocProducts($tecDocPairs, $order, $limit, $offset);

				$query->found_posts = $this->gath->getFoundRows();
				$query->max_num_pages = ceil($query->found_posts / $limit);

				return $this->loadPosts($products);
			}
		}

		if (
			'product' === $query->query_vars['post_type'] ||
			(is_array($query->query_vars['post_type']) && in_array('product', $query->query_vars['post_type']))
		) {
			if (count($query->query_vars['post__in']) > 0) {
				$productIds = [];

				foreach ($query->query_vars['post__in'] as $productId) {
					$productIds[] = $productId > GathService::PRODUCT_ID_OFFSET
						? $productId - GathService::PRODUCT_ID_OFFSET
						: $productId;
				}

				$products = $this->gath->getProductsByIds($productIds);
				return $this->loadPosts($products);
			}

			if (preg_match('/-(\d+)$/', $query->query_vars['name'], $matches)) {
				if ($post = $this->getProduct(intval($matches[1]))) {
					return [$post];
				}
			}
		} else if (!empty($query->query_vars['product_cat'])) {
			$query->set('no_found_rows', false);
			$limit = $query->query_vars['posts_per_page'];
			$page = max(1, intval($query->query_vars['paged']));
			$offset = ($page - 1) * $limit;

			$products = $this->gath->getProductsByCategory(get_queried_object_id(), $order, $limit, $offset);

			$query->found_posts = $this->gath->getFoundRows();
			$query->max_num_pages = ceil($query->found_posts / $limit);

			return $this->loadPosts($products);
		}

		return $null;
	}
}